import os
from pathlib import Path
from typing import Final

import pyperclip
import typer
from beni import bcolor, bfile, bfunc, binput, bpath, btask
from beni.btype import Null

from . import bin, venv

app: Final = btask.newSubApp('BTask 工具')


@app.command()
@bfunc.syncCall
async def create(
    projectPath: Path = typer.Argument(None, help="项目路径"),
):
    '创建 BTask 项目'
    if projectPath is None:
        projectPath = Path.cwd()
    if projectPath.exists():
        await binput.confirm(f'项目路径 {projectPath} 已存在，是否覆盖？')
    await bfile.makeFiles(_files, projectPath)
    init(projectPath)


@app.command()
@bfunc.syncCall
async def init(
    projectPath: Path = typer.Argument(None, help="项目路径"),
):
    '初始化 BTask 项目，包括 venv 和 bin 操作'
    if projectPath is None:
        projectPath = Path.cwd()
    venv.venv(
        packages=Null,
        path=projectPath,
        disabledMirror=False,
        quiet=True,
    )
    bin.download(
        names=Null,
        file=projectPath / 'bin.list',
        output=projectPath / 'bin',
    )


@app.command()
@bfunc.syncCall
async def export(
    projectPath: Path = typer.Argument(None, help="项目路径"),
):
    '导出文件内容到剪贴板'
    if projectPath is None:
        projectPath = Path.cwd()
    ignores = [
        projectPath / 'bin',
        projectPath / 'venv',
        projectPath / 'venv.lock',
    ]
    files = await bpath.listFile(projectPath, True)
    files = [x for x in files if '__pycache__' not in x.parts]
    contents: list[str] = []
    for file in files:
        for ignore in ignores:
            if file.is_relative_to(ignore):
                break
        else:
            contents.append(f'>>> {str(file.relative_to(projectPath)).replace(os.path.sep, "/")}\n{await bfile.readText(file)}')
    contents.sort()
    content = '_files = \'\'\'\n' + '\n\n\n'.join(contents) + '\n\'\'\''
    pyperclip.copy(content)
    bcolor.printGreen('已复制到剪贴板')


@app.command()
@bfunc.syncCall
async def tidy(
    tasksPath: Path = typer.Argument(None, help="tasks 路径"),
):
    '整理 tasks 文件'
    files = await bpath.listFile(tasksPath)
    files = [x for x in files if not x.name.startswith('_')]
    contents = [f'from . import {x.stem}' for x in files]
    contents.insert(0, '# type: ignore')
    contents.append('')
    await bfile.writeText(
        tasksPath / '__init__.py',
        '\n'.join(contents),
    )


_files = '''
>>> .gitignore
__pycache__
bin/
venv/


>>> bin.list



>>> src/.vscode/launch.json
{
    "version": "0.2.0",
    "configurations": [
        {
            "name": "debug",
            "type": "python",
            "request": "launch",
            // "console": "internalConsole",
            "program": "${workspaceFolder}/dev.py",
        }
    ],
}


>>> src/.vscode/settings.json
{
    "files.exclude": {
        "**/__pycache__": true,
        ".pytest_cache": true,
    },
    "python.defaultInterpreterPath": "${workspaceFolder}/../venv/Scripts/python.exe",
}


>>> src/.vscode/tasks.json
{
    "version": "2.0.0",
    "tasks": [
        {
            "label": "git commit",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:commit",
                "/path:${workspaceFolder}/../../",
            ],
        },
        {
            "label": "git commit file",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:commit",
                "/path:${file}",
            ],
        },
        {
            "label": "git pull",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:pull",
                "/path:${workspaceFolder}/../../",
            ],
        },
        {
            "label": "git revert",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:revert",
                "/path:${workspaceFolder}/../../",
            ],
        },
        {
            "label": "git sync",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:sync",
                "/path:${workspaceFolder}/../../",
            ],
        },
        {
            "label": "git log",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:log",
                "/path:${workspaceFolder}/../../",
            ],
        },
        {
            "label": "git log file",
            "type": "shell",
            "problemMatcher": [],
            "command": "TortoiseGitProc.exe",
            "args": [
                "/command:log",
                "/path:${file}",
            ],
        },
        {
            "label": "update version",
            "type": "shell",
            "problemMatcher": [],
            "command": "beni",
            "args": [
                "lib",
                "version",
                "${workspaceFolder}"
            ],
        },
    ],
}


>>> src/dev.py
import main
from beni import btask

main.init()

btask.dev('sub.haha')


>>> src/main.py
from beni import btask, bpath


def init():
    btask.options.tasksPath = bpath.get(__file__, '../tasks')
    btask.options.lock = 'xxxxxx'


def run():
    init()
    btask.main()


if __name__ == '__main__':
    run()


>>> src/tasks/__init__.py



>>> src/tasks/hello.py
from beni import bfunc, btask


@btask.app.command()
@bfunc.syncCall
async def hello():
    '打印 hello'
    print('hello')


>>> src/tasks/sub.py
from typing import Final

import typer
from beni import bfunc


app: Final = typer.Typer(help='子工具集')


@app.command()
@bfunc.syncCall
async def haha():
    '打印 haha'
    print('haha')


@app.command()
@bfunc.syncCall
async def bye():
    '打印 bye'
    print('bye')


>>> venv.list
aioconsole
aiofiles
aiohttp
aiosqlite
autopep8
benimang==0.4.10
nest-asyncio
orjson
portalocker
pretty_errors
prettytable
pydantic
pyinstaller
pyyaml
qiniu
typer
'''
